#This file is part of reparted.

#reparted is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#reparted is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with reparted.  If not, see <http://www.gnu.org/licenses/>.

from exception import SizeError

size_units = {
    "B":    1,       # byte
    "KB":   1000**1, # kilobyte
    "MB":   1000**2, # megabyte
    "GB":   1000**3, # gigabyte
    "TB":   1000**4, # terabyte
    "PB":   1000**5, # petabyte
    "EB":   1000**6, # exabyte
    "ZB":   1000**7, # zettabyte
    "YB":   1000**8, # yottabyte
    "KiB":  1024**1, # kibibyte
    "MiB":  1024**2, # mebibyte
    "GiB":  1024**3, # gibibyte
    "TiB":  1024**4, # tebibyte
    "PiB":  1024**5, # pebibyte
    "EiB":  1024**6, # exbibyte
    "ZiB":  1024**7, # zebibyte
    "YiB":  1024**8, # yobibyte
    "%":    1        # we've got percents!!!
}

class Size(object):
    """
    *Size class is used as a container to specify desired partition sizes.*

    You need to call this class before adding Partition instances.
    The default values for units is "MB" and sector_size is set to 512.
    You can manually specify the sector size or pass the device you intend
    to use it with and have it obtain the secdtor size from there::

        from reparted import *

        # sector size defaults to 512
        mySize1 = Size(4, "GB")

        # manually set the sector size to 1024
        mySize2 = Size(4, "GB", sector_size=1024)

        # get the sector size from the device
        myDevice = Device('/dev/sda')
        mySize = Size(4, "GB", myDevice)

    There is an option to choose percentage *'%'* as units, in which case
    you need to pass a Device instance to calculate the sectors needed
    and length should be between 1 and 100::

        from reparted import *

        myDevice = Device('/dev/sda')
        mySize = Size(25, "%", myDevice)

    *Args:*
        length (int):       The desired length.

        units (str):        The desired units (ie. "MB", "GB", "%" etc...)

        sector_size (int):  The sector size to be used to calculate.

        dev:                A Device instance. Only needed when using
                            percents, otherwise calculations will take
                            sector size from Device.sector_size.

    *Raises:*
        SizeError

    .. note::

       When using percent as unit and using the Size instance for multiple
       disks make sure they all have the same sector size.

    """
    def __init__(self, length, units="MB", sector_size=512, dev=None):
        self.__length = length
        self.units = units
        self.sector_size = getattr(dev, "sector_size", sector_size)
        if units != "%":
            self.sectors = (size_units[units] * length) / self.sector_size
        elif not dev:
            raise SizeError(400)
        elif not (0 < length <= 100) or type(length) is float:
            raise SizeError(401)
        else:
            self.sectors = (dev.length / 100) * length

    def to(self, units):
        """
        Returns the size in the specified units.

        *Args:*
            units (str):    The desired units (ie. "MB", "GB", "%" etc...)
        """
        sz = ((float(self.sectors) * self.sector_size) / size_units[units])
        return sz

    def pretty(self, units="MB"):
        """
        Returns the size in the specified units in a pretty string.

        *Args:*
            units (str):    The desired units (ie. "MB", "GB", "%" etc...)
        """
        sz = self.to(units)
        return "%.2f%s" % (sz, units)

    def __str__(self):
        return self.pretty()


#This file is part of reparted.

#reparted is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#reparted is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with reparted.  If not, see <http://www.gnu.org/licenses/>.

from ctypes.util import find_library
from ctypes import *

lib = find_library("parted")

if not lib:
    raise Exception("Parted library not found.")

parted = CDLL(lib)

class PedCHSGeometry(Structure):
    _fields_ = [
        ('cylinders', c_int),
        ('heads', c_int),
        ('sectors', c_int),
    ]

PedSector = c_longlong

class PedAlignment(Structure):
    _fields_ = [
        ('offset', PedSector),
        ('grain_size', PedSector),
     ]

class PedDevice(Structure):
    pass

PedDevice._fields_ = [
    ('next', POINTER(PedDevice)),
    ('model', c_char_p),
    ('path', c_char_p),
    ('type', c_int),
    ('sector_size', c_longlong),
    ('phys_sector_size', c_longlong),
    ('length', PedSector),
    ('open_count', c_int),
    ('read_only', c_int),
    ('external_mode', c_int),
    ('dirty', c_int),
    ('boot_dirty', c_int),
    ('hw_geom', PedCHSGeometry),
    ('bios_geom', PedCHSGeometry),
    ('host', c_short),
    ('did', c_short),
    ('arch_specific', c_void_p),
]

class PedGeometry(Structure):
    _fields_ = [
        ('dev', POINTER(PedDevice)),
        ('start', PedSector),
        ('length', PedSector),
        ('end', PedSector),
     ]

class PedPartition(Structure):
    pass

class PedDiskType(Structure):
    pass

class PedFileSystemType(Structure):
    pass

class PedDisk(Structure):
    _fields_ = [
        ('dev', POINTER(PedDevice)),
        ('type', POINTER(PedDiskType)),
        ('block_sizes', c_int),
        ('part_list', POINTER(PedPartition)),
        ('disk_specific', c_void_p),
        ('needs_clobber', c_int),
        ('update_mode', c_int),
     ]

class PedDiskOps(Structure):
    _fields_ = [
        ('probe', c_int),
        ('clobber', c_int),
        ('alloc', POINTER(PedDisk)),
        ('duplicate', POINTER(PedDisk)),
        ('free', c_void_p),
        ('read', c_int),
        ('write', c_int),
        ('disk_set_flag', c_int),
        ('disk_get_flag', c_int),
        ('disk_is_flag_available', c_int),
        ('partition_new', POINTER(PedPartition)),
        ('partition_duplicate', POINTER(PedPartition)),
        ('partition_destroy', c_void_p),
        ('partition_set_system', c_int),
        ('partition_set_flag', c_int),
        ('partition_get_flag', c_int),
        ('partition_is_flag_available', c_int),
        ('partition_set_name', c_void_p),
        ('partition_get_name', c_char_p),
        ('partition_align', c_int),
        ('partition_enumerate', c_int),
        ('partition_check', c_bool),
        ('alloc_metadata', c_int),
        ('get_max_primary_partition_count', c_int),
        ('get_max_supported_partition_count', c_bool),
        ('get_partition_alignment', POINTER(PedAlignment)),
        ('max_length', PedSector),
        ('max_start_sector', PedSector)
    ]

PedDiskType._fields_ = [
    ('next', POINTER(PedDiskType)),
    ('name', c_char_p),
    ('ops', POINTER(PedDiskOps)),
    ('features', c_int)
]

PedFileSystemType._fields_ = [
    ('next', POINTER(PedFileSystemType)),
    ('name', c_char_p),
    ('block_sizes', c_int),
    ('ops', POINTER(PedDiskOps)),
]

PedPartition._fields_ = [
    ('prev', POINTER(PedPartition)),
    ('next', POINTER(PedPartition)),
    ('disk', POINTER(PedDisk)),
    ('geom', PedGeometry),
    ('num', c_int),
    ('type', c_long),
    ('fs_type', POINTER(PedFileSystemType)),
    ('part_list', POINTER(PedPartition)),
    ('disk_specific', c_void_p),
]

class PedConstraint(Structure):
    _fields_ = [
        ('start_align', POINTER(PedAlignment)),
        ('end_align', POINTER(PedAlignment)),
        ('start_range', POINTER(PedGeometry)),
        ('end_range', POINTER(PedGeometry)),
        ('min_size', PedSector),
        ('max_size', PedSector),
     ]

# Device Function conversions
device_get = parted.ped_device_get
device_get.restype = POINTER(PedDevice)
device_get_constraint = parted.ped_device_get_constraint
device_get_constraint.restype = POINTER(PedConstraint)
device_get_optimal_aligned_constraint = parted.ped_device_get_optimal_aligned_constraint
device_get_optimal_aligned_constraint.argtypes = [POINTER(PedDevice)]
device_get_optimal_aligned_constraint.restype = POINTER(PedConstraint)
device_get_minimal_aligned_constraint = parted.ped_device_get_minimal_aligned_constraint
device_get_minimal_aligned_constraint.argtypes = [POINTER(PedDevice)]
device_get_minimal_aligned_constraint.restype = POINTER(PedConstraint)
device_get_optimum_alignment = parted.ped_device_get_optimum_alignment
device_get_optimum_alignment.argtypes = [POINTER(PedDevice)]
device_get_optimum_alignment.restype = POINTER(PedAlignment)
device_get_minimum_alignment = parted.ped_device_get_minimum_alignment
device_get_minimum_alignment.argtypes = [POINTER(PedDevice)]
device_get_minimum_alignment.restype = POINTER(PedAlignment)
device_get__constraint = parted.ped_device_get_constraint
device_get_constraint.argtypes = [POINTER(PedDevice)]
device_get_constraint.restype = POINTER(PedConstraint)

# Disk Function conversions
disk_probe = parted.ped_disk_probe
disk_probe.restype = POINTER(PedDiskType)
disk_new = parted.ped_disk_new
disk_new.restype = POINTER(PedDisk)
disk_new_fresh = parted.ped_disk_new_fresh
disk_new_fresh.argtypes = [POINTER(PedDevice), POINTER(PedDiskType)]
disk_new_fresh.restype = POINTER(PedDisk)
disk_add_partition = parted.ped_disk_add_partition
disk_add_partition.argtypes = [POINTER(PedDisk), POINTER(PedPartition), POINTER(PedConstraint)]
disk_next_partition = parted.ped_disk_next_partition
disk_next_partition.argtypes = [POINTER(PedDisk), POINTER(PedPartition)]
disk_next_partition.restype = POINTER(PedPartition)
disk_get_last_partition_num = parted.ped_disk_get_last_partition_num
disk_get_last_partition_num.argtypes = [POINTER(PedDisk)]
disk_get_partition = parted.ped_disk_get_partition
disk_get_partition.argtypes = [POINTER(PedDisk), c_int]
disk_get_partition.restype = POINTER(PedPartition)
disk_delete_partition = parted.ped_disk_delete_partition
disk_delete_partition.argtypes = [POINTER(PedDisk), POINTER(PedPartition)]
disk_delete_all = parted.ped_disk_delete_all
disk_delete_all.argtypes = [POINTER(PedDisk)]
disk_commit_to_os = parted.ped_disk_commit_to_os
disk_commit_to_os.argtypes = [POINTER(PedDisk)]
disk_commit_to_dev = parted.ped_disk_commit_to_dev
disk_commit_to_dev.argtypes = [POINTER(PedDisk)]
disk_destroy = parted.ped_disk_destroy
disk_destroy.argtypes = [POINTER(PedDisk)]
disk_destroy.restype = None
disk_get_type = parted.ped_disk_type_get
disk_get_type.restype = POINTER(PedDiskType)
disk_remove_partition = parted.ped_disk_remove_partition
disk_remove_partition.argtypes = [POINTER(PedDisk), POINTER(PedPartition)]

# Partition Function conversions
partition_new = parted.ped_partition_new
partition_new.argtypes = [POINTER(PedDisk), c_int, POINTER(PedFileSystemType), PedSector, PedSector]
partition_new.restype = POINTER(PedPartition)
partition_is_busy = parted.ped_partition_is_busy
partition_is_busy.argtypes = [POINTER(PedPartition)]
partition_get_name = parted.ped_partition_get_name
partition_get_name.argtypes = [POINTER(PedPartition)]
partition_get_name.restype = c_char_p
partition_set_name = parted.ped_partition_set_name
partition_set_name.argtypes = [POINTER(PedPartition), c_char_p]
partition_is_flag_available = parted.ped_partition_is_flag_available
partition_is_flag_available.argtypes = [POINTER(PedPartition), c_int]
partition_set_flag = parted.ped_partition_set_flag
partition_set_flag.argtypes = [POINTER(PedPartition), c_int, c_int]
geometry_new = parted.ped_geometry_new
geometry_new.argtypes = [POINTER(PedDevice), PedSector, PedSector]
geometry_new.restype = POINTER(PedGeometry)
constraint_new = parted.ped_constraint_new
constraint_new.argtypes = [POINTER(PedAlignment), POINTER(PedAlignment), POINTER(PedGeometry), POINTER(PedGeometry), PedSector, PedSector]
constraint_new.restype = POINTER(PedConstraint)
constraint_intersect = parted.ped_constraint_intersect
constraint_intersect.argtypes = [POINTER(PedConstraint), POINTER(PedConstraint)]
constraint_intersect.restype = POINTER(PedConstraint)
constraint_destroy = parted.ped_constraint_destroy
constraint_destroy.argtypes = [POINTER(PedConstraint)]
file_system_type_get = parted.ped_file_system_type_get
file_system_type_get.argtypes = [c_char_p]
file_system_type_get.restype = POINTER(PedFileSystemType)


class RepartedError(Exception):
    def __init__(self, code):
        self._register(code, code_dict=None)

    def _register(self, code, code_dict):
        self.code_dict = code_dict
        self.code = code

    def __str__(self):
        return repr(self.code_dict.get(self.code, None))

size_error_code = {
    400: "Device instance required for this operation.",
    401: "Invalid length type."
}

device_error_code = {
    500: "No device found."
}

disk_error_code = {
    600: "Failed to initialize disk.",
    601: "Failed to commit to device.",
    602: "Failed to commit to OS.",
    603: "Unsupported disk label.",
    604: "Failed to get disk type.",
    605: "Failed to create new disk.",
    606: "Method unavailable for initialized disk."
}

partition_error_code = {
    700: "Failed to initialize partition.",
    701: "Failed to add partition to disk.",
    702: "Failed to set user-defined constraint to disk.",
    703: "Failed to set device constraint to disk.",
    704: "Failed to set partition name to disk.",
    705: "Invalid partition instance or partition number.",
    706: "Partition is busy.",
    707: "Invalid partition type.",
    708: "Invalid alignment option.",
    709: "Invalid geometry.",
    710: "Unsupported flag.",
}

class SizeError(RepartedError):
    """
    Raised when a Size class error occurs for the following reasons:

    *       *Device instance required for this operation.*
    *       *Invalid length type.*

    """
    def __init__(self, code):
        self._register(code, size_error_code)

class DeviceError(RepartedError):
    """
    Raised when a Device class error occurs for the following reasons:

    *       *No device found.*

    """
    def __init__(self, code):
        self._register(code, device_error_code)

class DiskError(RepartedError):
    """
    Raised when a Disk class error occurs for the following reasons:

    *       *Failed to initialize disk.*
    *       *Unsupported disk label.*
    *       *Failed to get disk type.*
    *       *Failed to create new disk.*
    *       *Method unavailable for initialized disk.*

    """
    def __init__(self, code):
        self._register(code, disk_error_code)

class DiskCommitError(RepartedError):
    """
    Raised when the Disk commit method fails for the following reasons:

    *       *Failed to commit to device.*
    *       *Failed to commit to OS.*

    """
    def __init__(self, code):
        self._register(code, disk_error_code)

class PartitionError(RepartedError):
    """
    Raised when a Partition class error occurs for the following reasons:

    *       *Failed to initialize partition.*
    *       *Failed to set partition name to disk.*
    *       *Invalid partition instance or partition number.*
    *       *Invalid partition type.*
    *       *Invalid alignment option.*
    *       *Invalid geometry.*
    *       *Unsupported flag.*

    """
    def __init__(self, code):
        self._register(code, partition_error_code)

class AddPartitionError(RepartedError):
    """
    Raised when the Disk add_partition method fails for the following reasons:

    *       *Failed to add partition to disk.*
    *       *Failed to set user-defined constraint to disk.*
    *       *Failed to set device constraint to disk.*
    *       *Failed to set partition name to disk.*

    """
    def __init__(self, code):
        self._register(code, partition_error_code)

class DeletePartitionError(RepartedError):
    """
    Raised when the Disk delete_partition method fails for the following reasons:

    *       *Invalid partition instance or partition number.*
    *       *Partition is busy.*

    """
    def __init__(self, code):
        self._register(code, partition_error_code)


#This file is part of reparted.

#reparted is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#reparted is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with reparted.  If not, see <http://www.gnu.org/licenses/>.

from conversion import *
from disk import *
from exception import DeviceError
import os

device_type = {
    0 : 'UNKNOWN',
    1 : 'SCSI',
    2 : 'IDE',
    3 : 'DAC960',
    4 : 'CPQARRAY',
    5 : 'FILE',
    6 : 'ATARAID',
    7 : 'I20',
    8 : 'UBD',
    9 : 'DASD',
    10 : 'VIODASD',
    11 : 'SX8',
    12 : 'DM',
    13 : 'XVD',
    14 : 'SDMMC',
    15 : 'VIRTBLK',
    16 : 'AOE',
    17 : 'MD',
}

standard_devices = [
    "/dev/hda",
    "/dev/hdb",
    "/dev/hdc",
    "/dev/hdd",
    "/dev/hde",
    "/dev/hdf",
    "/dev/hdg",
    "/dev/hdh",
    "/dev/sda",
    "/dev/sdb",
    "/dev/sdc",
    "/dev/sdd",
    "/dev/sde",
    "/dev/sdf"
]

def device_probe(path):
    if not os.path.exists(path):
        return False
    dev = device_get(path)
    if bool(dev):
        return dev
    else:
        return False

class Device(object):
    """
    *Device class is used as a wrapper to libparted's ped_device.*

    You need to call this class before calling Disk. You can create
    a new instance of Device by specifying a path or let it probe
    standard devices::

        from reparted import *

        # specify the device path
        myDevice = Device("/dev/sdb")

        # probe standard devices
        myDevice2 = Device()

    *Args:*
        path (str):     Path to your local device of choice (ie. '/dev/sda').
        dev:            A ped_device pointer, usually this is used internally.

    *Raises:*
        DeviceError

    .. note::

       If called without any parameters it will probe all standard devices
       and default to the first one it finds.

    """
    def __init__(self, path=None, dev=None):
        """
        Initialize Device class, if ped_device pointer is Null it will
        raise DeviceError.
        """
        if path:
            self.__device = device_probe(path)
        elif dev:
            self.__device = dev
        else:
            self.__device = self._probe_ped_device()
        if not bool(self.__device):
            raise DeviceError(500)

    @property
    def _ped_device(self):
        """
        Returns the ctypes ped_device pointer.
        """
        return self.__device

    @property
    def length(self):
        """
        Returns the length in sectors of the device.
        """
        return self.__device.contents.length

    @property
    def path(self):
        """
        Returns the device path (ie. '/dev/sda').
        """
        return self.__device.contents.path

    @property
    def model(self):
        """
        Returns the device model (ie. 'ATA VBOX HARDDISK').
        """
        return self.__device.contents.model

    @property
    def type(self):
        """
        Returns the device type (ie. 'SCSI').
        """
        return device_type[self.__device.contents.type]

    @property
    def sector_size(self):
        return self.__device.contents.sector_size

    @property
    def phys_sector_size(self):
        """
        Returns the physical sector size.
        """
        return self.__device.contents.phys_sector_size

    @property
    def open_count(self):
        """
        Returns the number of times the device has been opened.
        """
        return self.__device.contents.open_count

    @property
    def read_only(self):
        """
        Returns True if the device is set as read only.
        """
        return bool(self.__device.contents.read_only)

    @property
    def external_mode(self):
        """
        Returns True if the device is set to external mode.
        """
        return bool(self.__device.contents.path)

    @property
    def dirty(self):
        """
        Returns True if the device is dirty.
        """
        return bool(self.__device.contents.dirty)

    @property
    def boot_dirty(self):
        """
        Returns True if the device is set to boot dirty.
        """
        return bool(self.__device.contents.boot_dirty)

    @property
    def hw_geom(self):
        """
        Returns the hardware geometry as a 3-tuple:

            (cylinders, heads, sectors)
        """
        cylinders = self.__device.contents.hw_geom.cylinders
        heads = self.__device.contents.hw_geom.heads
        sectors = self.__device.contents.hw_geom.sectors
        return (cylinders, heads, sectors)

    @property
    def bios_geom(self):
        """
        Returns the bios geometry as a 3-tuple:

            (cylinders, heads, sectors)
        """
        cylinders = self.__device.contents.bios_geom.cylinders
        heads = self.__device.contents.bios_geom.heads
        sectors = self.__device.contents.bios_geom.sectors
        return (cylinders, heads, sectors)

    @property
    def host(self):
        """
        Returns the device host.
        """
        return self.__device.contents.host

    @property
    def did(self):
        """
        Returns the device did.
        """
        return self.__device.contents.did

    def _probe_ped_device(self):
        for path in standard_devices:
            dev = device_probe(path)
            if bool(dev):
                return dev
        raise DeviceError(500)

def probe_standard_devices():
    """
    This function probes all standard devices and returns a list
    containing instances of Device of the found standard devices::

        from reparted.device import probe_standard_devices

        probe_standard_devices()
        [<reparted.device.Device object at 0xb7854d8c>,
        <reparted.device.Device object at 0xb7607eac>]
    """
    devices = []
    for path in standard_devices:
        dev = device_probe(path)
        if dev:
            device = Device(dev=dev)
            devices.append(device)
    return devices


#!/usr/bin/env python

from distutils.core import setup

setup(name='reparted',
      version='1.1',
      description='Python libparted bindings',
      author='Ruben Quinones',
      author_email='rq.sysadmin@gmail.com',
      url='http://github.com/xzased/reparted',
      packages=['reparted'],
     )


#This file is part of reparted.

#reparted is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#reparted is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with reparted.  If not, see <http://www.gnu.org/licenses/>.

from conversion import *
from exception import *
import os

disk_features = {
    1 : 'EXTENDED',
    2 : 'PARTITION_NAME'
}

disk_labels = [
    "gpt",
    "msdos"
]

alignment_any = PedAlignment(0, 1)

partition_type = {
    0 : 'NORMAL',
    1 : 'LOGICAL',
    2 : 'EXTENDED',
    4 : 'FREESPACE',
    8 : 'METADATA',
    10 : 'PROTECTED'
}

partition_flag = {
    "BOOT" : 1,
    "ROOT" : 2,
    "SWAP" : 3,
    "HIDDEN" : 4,
    "RAID" : 5,
    "LVM" : 6,
    "LBA" : 7,
    "HPSERVICE" : 8,
    "PALO" : 9,
    "PREP" : 10,
    "MSFT_RESERVED" : 11,
    "BIOS_GRUB" : 12,
    "APPLE_TV_RECOVERY" : 13,
    "DIAG" : 14,
    "LEGACY_BOOT" : 15
}

def diskDecorator(error=False):
    def wrap(fn):
        def wrapped(self, *args, **kwargs):
            if bool(self._ped_disk):
                return fn(self, *args, **kwargs)
            if error:
                raise DiskError(606)
            return None
        return wrapped
    return wrap

class Disk(object):
    """
    *Disk class is used as a wrapper to libparted's ped_disk.*

    You need to call this class to list, add or delete partitions.
    A new instance of Disk is initialized by passing the Device instance::

        from reparted import *

        myDevice = Device("/dev/sda")
        myDisk = Disk(myDevice)

    *Args:*
        dev:    A Device class instance.

        disk:   A ped_disk pointer, usually this is used internally.

    *Raises:*
        DiskError

    .. note::

       You need to pass either a Device instance or a ped_disk pointer.
       If a disk is being initialized (no partition table) only the
       set_label method is available, with other methods returning
       None or raising DiskError.
    """
    def __init__(self, device=None, disk=None):
        if device:
            self.__device = device._ped_device
            self.__disk = disk_new(self.__device)
        elif disk:
            self.__disk = disk
            self.__device = self.__disk.contents.dev
        else:
            raise DiskError(600)

    @property
    def _ped_device(self):
        """
        Returns the ctypes ped_device pointer.
        """
        return self.__device

    @property
    def _ped_disk(self):
        """
        Returns the ctypes ped_disk pointer.
        """
        return self.__disk

    @property
    @diskDecorator()
    def type_name(self):
        """
        Returns the disk type (ie. 'gpt' or 'msdos').
        """
        return self.__disk.contents.type.contents.name

    @property
    @diskDecorator()
    def device(self):
        """
        Returns the Device the disk belongs to.
        """
        return self.__device

    @property
    @diskDecorator()
    def type_features(self):
        """
        Returns the features available (ie. 'EXTENDED' for
        lvm and 'PARTITION_NAME' for label support).
        """
        feat = self.__disk.contents.type.contents.features
        return disk_features[feat]

    @property
    @diskDecorator()
    def block_sizes(self):
        """
        Returns the disk block sizes.
        """
        return self.__disk.contents.block_sizes

    @property
    @diskDecorator()
    def needs_clobber(self):
        """
        Returns True if the disk needs clobber.
        """
        return bool(self.__disk.contents.needs_clobber)

    @property
    @diskDecorator()
    def update_mode(self):
        """
        Returns True if the disk is set to update mode.
        """
        return bool(self.__disk.contents.update_mode)

    @diskDecorator()
    def partitions(self):
        """
        Returns a list of the current disk partitions.

        .. note::

            If the disk is initialized (no partition table) it
            will return None, if the disk has a partition table
            but no partitions it will return an empty list.
        """
        partitions = []
        part = disk_next_partition(self.__disk, None)
        while part:
            if part.contents.type > 2:
                part = disk_next_partition(self.__disk, part)
                continue
            p = Partition(part=part)
            partitions.append(p)
            part = disk_next_partition(self.__disk, part)
        return partitions

    @diskDecorator(error=True)
    def add_partition(self, part):
        """
        Adds a partition to disk. You still need to call commit
        for the changes to be made to disk::

            from reparted import *

            myDevice = Device("/dev/sdb")
            myDisk = Disk(myDevice)
            mySize = Size(4, "GB")
            myPartition = Partition(myDisk, mySize)
            myDisk.add_partition(myPartition)
            myDisk.commit()

        *Args:*
            part:       A Partition class instance.

        *Raises:*
            AddPartitionError

        .. note::

            If the disk is initialized (no partition table) it
            will raise DiskError.
        """
        try:
            p = self.get_partition(part.num)
            if p.geom == part.geom:
                raise AddPartitionError(701)
        except ValueError:
            pass
        partition = part._Partition__partition
        disk = self.__disk
        start, end, length = part.geom
        range_start = geometry_new(self.__device, start, 1)
        range_end = geometry_new(self.__device, end, 1)
        user_constraint = constraint_new(alignment_any, alignment_any, range_start, range_end, 1, disk.contents.dev.contents.length)
        if not bool(user_constraint):
            raise AddPartitionError(702)
        if part.alignment == 'optimal':
            dev_constraint = device_get_optimal_aligned_constraint(self.__device)
        elif part.alignment == 'minimal':
            dev_constraint = device_get_minimal_aligned_constraint(self.__device)
        else:
            dev_constraint = device_get_constraint(self.__device)
        if not bool(dev_constraint):
            raise AddPartitionError(702)
        final_constraint = constraint_intersect(user_constraint, dev_constraint)
        constraint_destroy(user_constraint)
        constraint_destroy(dev_constraint)
        if not bool(final_constraint):
            raise AddPartitionError(703)
        added = disk_add_partition(disk, partition, final_constraint)
        constraint_destroy(final_constraint)
        if not added:
            disk_remove_partition(disk, partition)
            raise AddPartitionError(701)
        if part.name:
            set_name = partition_set_name(partition, part.name)
            if not set_name:
                disk_remove_partition(disk, partition)
                raise AddPartitionError(704)

    @diskDecorator(error=True)
    def delete_partition(self, part):
        """
        Deletes a partition from disk. Unlike add_partition,
        this method calls commit, use carefully::

            from reparted import *

            myDevice = Device("/dev/sdb")
            myDisk = Disk(myDevice)

            # Get Partition instance and remove it.
            part = myDisk.partitions()[0]
            myDisk.delete_partition(part)

            # Get Partition number and remove it.
            part = myDisk.partitions()[0].num
            myDisk.delete_partition(part)

        *Args:*
            part:   A Partition class instance OR partition number.

        *Raises:*
            DeletePartitionError, DiskCommitError

        .. note::

            If the disk is initialized (no partition table) it
            will raise DiskError.
        """
        if part and isinstance(part, Partition):
            partition = part._Partition__partition
        elif type(part) is int:
            partition = self._get_ped_partition(part)
        else:
            raise DeletePartitionError(705)
        if partition_is_busy(partition):
            raise DeletePartitionError(706)
        disk_delete_partition(self.__disk, partition)
        self.commit()
        disk_destroy(self.__disk)
        self.__disk = disk_new(self.__device)

    @diskDecorator()
    def delete_all(self):
        """
        This method deletes all partitions from disk.

        *Raises:*
            DiskError, DiskCommitError

        .. note::

            If the disk is initialized (no partition table) it
            will return None.
        """
        disk_delete_all(self.__disk)
        return

    @diskDecorator(error=True)
    def commit(self):
        """
        This method commits partition modifications to disk.

        *Raises:*
            DiskError, DiskCommitError

        .. note::

            If the disk is initialized (no partition table) it
            will return None.
        """
        to_dev = disk_commit_to_dev(self.__disk)
        if not to_dev:
            raise DiskCommitError(601)
        to_os = disk_commit_to_os(self.__disk)
        if not to_os:
            raise DiskCommitError(602)

    def _get_ped_partition(self, part_num):
        partition = disk_get_partition(self.__disk, part_num)
        if not bool(partition):
            raise PartitionError(705)
        return partition

    @diskDecorator(error=True)
    def get_partition(self, part_num):
        """
        Returns a Partition instance.

        *Args:*
            part_num (int):     A partition number.

        *Raises:*
            PartitionError

        .. note::

            If the disk is initialized (no partition table) it
            will raise DiskError.
        """
        partition = Partition(part=self._get_ped_partition(part_num))
        return partition

    def _destroy_disk(self, disk=None):
        if disk:
            disk_destroy(disk)
        else:
            if self.__disk:
                disk_destroy(self._ped_disk)
                self.__disk = None
            else:
                raise DiskError(600)

    def set_label(self, label):
        """
        Sets the disk partition table ('gpt' or 'msdos)'.
        This method calls commit to set the label changes.

        *Args:*
            label (str):    A partition table type ('gpt' or 'msdos')

        *Raises:*
            DiskError
        """
        if label not in disk_labels:
            raise DiskError(603)
        disk_type = disk_get_type(label)
        if not bool(disk_type):
            raise DiskError(604)
        if bool(self._ped_disk):
            self._destroy_disk()
        new_disk = disk_new_fresh(self._ped_device, disk_type)
        if not bool(new_disk):
            raise DiskError(605)
        self.__disk = new_disk
        self.commit()
        self._destroy_disk(disk=new_disk)
        self.__disk = disk_new(self.__device)

class Partition(object):
    """
    *Partition class is used as a wrapper to libparted's ped_partition.*

    You need can create Partition instances and add them to disk. A new
    Partition instance can be initialized::

        from reparted import *

        myDevice = Device("/dev/sda")
        myDisk = Disk(myDevice)
        mySize = Size(4, "GB")

        # Defaults
        myPartition = Partition(myDisk, mySize)

        # Different filesystem and minimal alignment.
        myPartition = Partition(myDisk, mySize, fs="ext4", align="minimal")

        # Initialize with a name
        if myDisk.type_features == 'PARTITION_NAME':
            myPartition = Partition(myDisk, mySize, name="test")

    *Args:*
        disk:           A Disk class instance.
        size:           A Size class instance.
        type (str):     The partition type (ie. 'NORMAL', 'LOGICAL', etc...).
        fs (str):       The filesystem type (ie. 'ext3', 'ext4', etc...).
        align (str):    The partition alignment, 'minimal' or 'optimal'.
        name (str):     The partition name.
        start (int):    The start sector for the partition.
        end (int):      The end sector for the partition.
        part:           A ctypes ped_partition pointer.

    *Raises:*
        PartitionError

    .. note::

       Name is only available when partition type is 'NORMAL'.
       The start and end arguments are optional and you should only use them when
       your want to specify such attributes, otherwise use optimal alignment.
       The part argument is optional and mostly for internal use.
    """
    def __init__(self, disk=None, size=None, type='NORMAL', fs='ext3', align='optimal',
                    name='', start=None, end=None, part=None):
        if part:
            self.__align = None
            self.__partition = part
            self.__ped_disk = self.__partition.contents.disk
            self.__disk = Disk(disk=self.__ped_disk)
        elif disk and size:
            self.__align = align
            if not type in partition_type.values():
                raise PartitionError(707)
            part_type = [key for key,val in partition_type.iteritems() if val == type][0]
            if type != 'EXTENDED' and fs != None:
                filesystem = file_system_type_get(fs)
            self.__ped_disk = disk._ped_disk
            self.__disk = Disk(disk=self.__ped_disk)
            if align == 'optimal' or align == 'minimal':
                dev = disk._ped_device
                a_start, a_end = self._get_alignment(dev, align, start, end, size)
            else:
                raise PartitionError(708)
            self.__partition = partition_new(self.__ped_disk, part_type, filesystem, a_start, a_end)
            if name:
                self.set_name(name)
        else:
            raise PartitionError(700)

    @property
    def disk(self):
        """
        Returns the ctypes ped_disk pointer.
        """
        return self.__disk

    @property
    def device(self):
        """
        Returns the Device instance this partition belongs to.
        """
        return self.disk.device

    @property
    def geom(self):
        """
        Returns the partition geometry as a 3-tuple:

            (start, end, length)
        """
        start =  self.__partition.contents.geom.start
        end =  self.__partition.contents.geom.end
        length =  self.__partition.contents.geom.length
        return (start, end, length)

    @property
    def num(self):
        """
        Returns the partition number.
        """
        return self.__partition.contents.num

    @property
    def type(self):
        """
        Returns the partition type.
        """
        return partition_type[self.__partition.contents.type]

    @property
    def fs_type(self):
        """
        Returns the partition filesystem type.
        """
        try:
            fs = self.__partition.contents.fs_type.contents.name
        except ValueError:
            fs = None
        return fs

    @property
    def name(self):
        """
        Returns the partition name if names are supported by disk type,
        otherwise returns None.
        """
        if self.disk.type_features != 'PARTITION_NAME':
            return None
        return partition_get_name(self.__partition)

    @property
    def alignment(self):
        """
        Returns the partition alignment ('optimal' or 'minimal').

        .. note::

            If you specify a 'minimal' alignment when creating a partition
            but the start sector falls in what would be considered an
            optimal alignment this method will return 'optimal'.
        """
        if self.__align:
            return self.__align
        else:
            optimal = device_get_optimum_alignment(self.disk._ped_device)
            minimal = device_get_minimum_alignment(self.disk._ped_device)
            start, e, l = self.geom
            if start % optimal.contents.grain_size == optimal.contents.offset:
                return 'optimal'
            if start % minimal.contents.grain_size == minimal.contents.offset:
                return 'minimal'
        return None

    def set_name(self, name):
        """
        Sets the partition name. If the disk type does not support
        partition names it will raise NotImplementedError.

        *Args:*
            name (str):         The partition name.

        *Raise:*
            NotImplementedError, PartitionError
        """
        if self.disk.type_features != 'PARTITION_NAME':
            raise NotImplementedError("The disk does not support partition names.")
        new_name = partition_set_name(self.__partition, name)
        if not new_name:
            raise PartitionError(704)
        return

    def _snap_sectors(self, start, end, size):
        if start:
            if not end:
                end = start + size.sectors - 1
            if (end - start) != (size.sectors - 1):
                raise PartitionError(709)
        else:
            last_part_num = disk_get_last_partition_num(self.disk._ped_disk)
            last_part = disk_get_partition(self.disk._ped_disk, last_part_num)
            last_end_sector = last_part.contents.geom.end
            start = last_end_sector + 1
            end = start + size.sectors - 1
        return (start, end)

    def _get_alignment(self, dev, align, start, end, size):
        const = getattr(parted, "ped_device_get_%s_aligned_constraint" % align)
        constraint = const(dev)
        start_offset = constraint.contents.start_align.contents.offset
        start_grain = constraint.contents.start_align.contents.grain_size
        end_offset = constraint.contents.end_align.contents.offset
        end_grain = constraint.contents.end_align.contents.grain_size
        snap_start, snap_end = self._snap_sectors(start, end, size)
        if snap_start % start_grain == start_offset:
            start = snap_start
        else:
            start = ((snap_start / start_grain) + 1) * start_grain
        end = start + size.sectors
        if (end - end_offset) % end_grain != end_offset:
            end = ((end / end_grain) * end_grain) + end_offset
        return (start, end)

    def _get_percent_size(self, length):
        device_length = self.device.length
        return

    def _check_flag(self, flag):
        if not flag in partition_flag.keys():
            raise PartitionError(710)
        check = partition_is_flag_available(self.__partition, partition_flag[flag])
        if not check:
            raise PartitionError(710)

    def set_flag(self, flag, state):
        """
        Sets the partition flag (ie. 'BOOT', 'ROOT', etc...).

        *Args:*
            flag (str):         The partition flag.
            state (bool):       Toggle the flag state (True or False).
        """
        self._check_flag(flag)
        partition_set_flag(self.__partition, partition_flag[flag], int(state))


#This file is part of reparted.

#reparted is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#reparted is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with reparted.  If not, see <http://www.gnu.org/licenses/>.

from .size import Size
from .device import Device
from .disk import Disk, Partition
